@@ -12,7 +12,7 @@
 ** This file contains C code routines that are called by the parser
 ** to handle INSERT statements in SQLite.
 **
-** $Id: insert.c,v 1.7 2002/04/02 10:43:03 matt Exp $
+** $Id: insert.c,v 1.8 2002/06/17 23:46:21 matt Exp $
 */
 #include "sqliteInt.h"
 
@@ -40,7 +40,7 @@ void sqliteInsert(
   int onError           /* How to handle constraint errors */
 ){
   Table *pTab;          /* The table to insert into */
-  char *zTab;           /* Name of the table into which we are inserting */
+  char *zTab = 0;       /* Name of the table into which we are inserting */
   int i, j, idx;        /* Loop counters */
   Vdbe *v;              /* Generate code into this virtual machine */
   Index *pIdx;          /* For looping over indices of the table */
@@ -53,6 +53,9 @@ void sqliteInsert(
   int keyColumn = -1;   /* Column that is the INTEGER PRIMARY KEY */
   int endOfLoop;        /* Label for the end of the insertion loop */
 
+  int row_triggers_exist = 0; /* True if there are FOR EACH ROW triggers */
+  int newIdx = -1;
+
   if( pParse->nErr || sqlite_malloc_failed ) goto insert_cleanup;
   db = pParse->db;
 
@@ -60,19 +63,43 @@ void sqliteInsert(
   */
   zTab = sqliteTableNameFromToken(pTableName);
   if( zTab==0 ) goto insert_cleanup;
-  pTab = sqliteTableNameToTable(pParse, zTab);
+  pTab = sqliteFindTable(pParse->db, zTab);
+  if( pTab==0 ){
+    sqliteSetString(&pParse->zErrMsg, "no such table: ", zTab, 0);
+    pParse->nErr++;
+    goto insert_cleanup;
+  }
+
+  /* Ensure that:
+  *  (a) the table is not read-only, 
+  *  (b) that if it is a view then ON INSERT triggers exist
+  */
+  row_triggers_exist = 
+    sqliteTriggersExist(pParse, pTab->pTrigger, TK_INSERT, 
+        TK_BEFORE, TK_ROW, 0) ||
+    sqliteTriggersExist(pParse, pTab->pTrigger, TK_INSERT, TK_AFTER, TK_ROW, 0);
+  if( pTab->readOnly || (pTab->pSelect && !row_triggers_exist) ){
+    sqliteSetString(&pParse->zErrMsg, 
+      pTab->pSelect ? "view " : "table ",
+      zTab,
+      " may not be modified", 0);
+    pParse->nErr++;
+    goto insert_cleanup;
+  }
   sqliteFree(zTab);
+  zTab = 0;
+
   if( pTab==0 ) goto insert_cleanup;
-  assert( pTab->pSelect==0 );  /* This table is not a VIEW */
 
   /* Allocate a VDBE
   */
   v = sqliteGetVdbe(pParse);
   if( v==0 ) goto insert_cleanup;
-  if( pSelect ){
-    sqliteBeginMultiWriteOperation(pParse);
-  }else{
-    sqliteBeginWriteOperation(pParse);
+  sqliteBeginWriteOperation(pParse, pSelect || row_triggers_exist);
+
+  /* if there are row triggers, allocate a temp table for new.* references. */
+  if( row_triggers_exist ){
+    newIdx = pParse->nTab++;
   }
 
   /* Figure out how many columns of data are supplied.  If the data
@@ -90,16 +117,19 @@ void sqliteInsert(
     assert( pSelect->pEList );
     nColumn = pSelect->pEList->nExpr;
   }else{
-    IdList dummy;
+    SrcList dummy;
     assert( pList!=0 );
     srcTab = -1;
     assert( pList );
     nColumn = pList->nExpr;
-    dummy.nId = 0;
+    dummy.nSrc = 0;
     for(i=0; i<nColumn; i++){
       if( sqliteExprResolveIds(pParse, 0, &dummy, 0, pList->a[i].pExpr) ){
         goto insert_cleanup;
       }
+      if( sqliteExprCheck(pParse, pList->a[i].pExpr, 0, 0) ){
+        goto insert_cleanup;
+      }
     }
   }
 
@@ -170,18 +200,29 @@ void sqliteInsert(
     keyColumn = pTab->iPKey;
   }
 
-  /* Open cursors into the table that is received the new data and
-  ** all indices of that table.
+  /* Open the temp table for FOR EACH ROW triggers */
+  if( row_triggers_exist ){
+    sqliteVdbeAddOp(v, OP_OpenTemp, newIdx, 0);
+  }
+    
+  /* Initialize the count of rows to be inserted
   */
-  base = pParse->nTab;
-  openOp = pTab->isTemp ? OP_OpenWrAux : OP_OpenWrite;
-  sqliteVdbeAddOp(v, openOp, base, pTab->tnum);
-  sqliteVdbeChangeP3(v, -1, pTab->zName, P3_STATIC);
-  for(idx=1, pIdx=pTab->pIndex; pIdx; pIdx=pIdx->pNext, idx++){
-    sqliteVdbeAddOp(v, openOp, idx+base, pIdx->tnum);
-    sqliteVdbeChangeP3(v, -1, pIdx->zName, P3_STATIC);
+  if( db->flags & SQLITE_CountRows && !pParse->trigStack ){
+    sqliteVdbeAddOp(v, OP_Integer, 0, 0);  /* Initialize the row count */
+  }
+
+  /* Open tables and indices if there are no row triggers */
+  if( !row_triggers_exist ){
+    base = pParse->nTab;
+    openOp = pTab->isTemp ? OP_OpenWrAux : OP_OpenWrite;
+    sqliteVdbeAddOp(v, openOp, base, pTab->tnum);
+    sqliteVdbeChangeP3(v, -1, pTab->zName, P3_STATIC);
+    for(idx=1, pIdx=pTab->pIndex; pIdx; pIdx=pIdx->pNext, idx++){
+      sqliteVdbeAddOp(v, openOp, idx+base, pIdx->tnum);
+      sqliteVdbeChangeP3(v, -1, pIdx->zName, P3_STATIC);
+    }
+    pParse->nTab += idx;
   }
-  pParse->nTab += idx;
 
   /* If the data source is a SELECT statement, then we have to create
   ** a loop because there might be multiple rows of data.  If the data
@@ -189,80 +230,137 @@ void sqliteInsert(
   ** and the loop is not used.
   */
   if( srcTab>=0 ){
-    if( db->flags & SQLITE_CountRows ){
-      sqliteVdbeAddOp(v, OP_Integer, 0, 0);  /* Initialize the row count */
-    }
     iBreak = sqliteVdbeMakeLabel(v);
     sqliteVdbeAddOp(v, OP_Rewind, srcTab, iBreak);
     iCont = sqliteVdbeCurrentAddr(v);
   }
 
+  endOfLoop = sqliteVdbeMakeLabel(v);
+  if( row_triggers_exist ){
+
+    /* build the new.* reference row */
+    sqliteVdbeAddOp(v, OP_Integer, 13, 0);
+    for(i=0; i<pTab->nCol; i++){
+      if( pColumn==0 ){
+        j = i;
+      }else{
+        for(j=0; j<pColumn->nId; j++){
+          if( pColumn->a[j].idx==i ) break;
+        }
+      }
+      if( pColumn && j>=pColumn->nId ){
+        sqliteVdbeAddOp(v, OP_String, 0, 0);
+        sqliteVdbeChangeP3(v, -1, pTab->aCol[i].zDflt, P3_STATIC);
+      }else if( srcTab>=0 ){
+        sqliteVdbeAddOp(v, OP_Column, srcTab, j); 
+      }else{
+        sqliteExprCode(pParse, pList->a[j].pExpr);
+      }
+    }
+    sqliteVdbeAddOp(v, OP_MakeRecord, pTab->nCol, 0);
+    sqliteVdbeAddOp(v, OP_PutIntKey, newIdx, 0);
+    sqliteVdbeAddOp(v, OP_Rewind, newIdx, 0);
+
+    /* Fire BEFORE triggers */
+    if( sqliteCodeRowTrigger(pParse, TK_INSERT, 0, TK_BEFORE, pTab, newIdx, -1, 
+        onError, endOfLoop) ){
+      goto insert_cleanup;
+    }
+
+    /* Open the tables and indices for the INSERT */
+    if( !pTab->pSelect ){
+      base = pParse->nTab;
+      openOp = pTab->isTemp ? OP_OpenWrAux : OP_OpenWrite;
+      sqliteVdbeAddOp(v, openOp, base, pTab->tnum);
+      sqliteVdbeChangeP3(v, -1, pTab->zName, P3_STATIC);
+      for(idx=1, pIdx=pTab->pIndex; pIdx; pIdx=pIdx->pNext, idx++){
+        sqliteVdbeAddOp(v, openOp, idx+base, pIdx->tnum);
+        sqliteVdbeChangeP3(v, -1, pIdx->zName, P3_STATIC);
+      }
+      pParse->nTab += idx;
+    }
+  }
+
   /* Push the record number for the new entry onto the stack.  The
   ** record number is a randomly generate integer created by NewRecno
   ** except when the table has an INTEGER PRIMARY KEY column, in which
   ** case the record number is the same as that column. 
   */
-  if( keyColumn>=0 ){
-    if( srcTab>=0 ){
-      sqliteVdbeAddOp(v, OP_Column, srcTab, keyColumn);
-    }else{
-      int addr;
-      sqliteExprCode(pParse, pList->a[keyColumn].pExpr);
+  if( !pTab->pSelect ){
+    if( keyColumn>=0 ){
+      if( srcTab>=0 ){
+        sqliteVdbeAddOp(v, OP_Column, srcTab, keyColumn);
+      }else{
+        sqliteExprCode(pParse, pList->a[keyColumn].pExpr);
 
-      /* If the PRIMARY KEY expression is NULL, then use OP_NewRecno
-      ** to generate a unique primary key value.
-      */
-      addr = sqliteVdbeAddOp(v, OP_Dup, 0, 1);
-      sqliteVdbeAddOp(v, OP_NotNull, 0, addr+4);
-      sqliteVdbeAddOp(v, OP_Pop, 1, 0);
+        /* If the PRIMARY KEY expression is NULL, then use OP_NewRecno
+        ** to generate a unique primary key value.
+        */
+        sqliteVdbeAddOp(v, OP_NotNull, -1, sqliteVdbeCurrentAddr(v)+3);
+        sqliteVdbeAddOp(v, OP_Pop, 1, 0);
+        sqliteVdbeAddOp(v, OP_NewRecno, base, 0);
+      }
+      sqliteVdbeAddOp(v, OP_MustBeInt, 0, 0);
+    }else{
       sqliteVdbeAddOp(v, OP_NewRecno, base, 0);
     }
-    sqliteVdbeAddOp(v, OP_MustBeInt, 0, 0);
-  }else{
-    sqliteVdbeAddOp(v, OP_NewRecno, base, 0);
-  }
 
-  /* Push onto the stack, data for all columns of the new entry, beginning
-  ** with the first column.
-  */
-  for(i=0; i<pTab->nCol; i++){
-    if( i==pTab->iPKey ){
-      /* The value of the INTEGER PRIMARY KEY column is always a NULL.
-      ** Whenever this column is read, the record number will be substituted
-      ** in its place.  So will fill this column with a NULL to avoid
-      ** taking up data space with information that will never be used. */
-      sqliteVdbeAddOp(v, OP_String, 0, 0);
-      continue;
-    }
-    if( pColumn==0 ){
-      j = i;
-    }else{
-      for(j=0; j<pColumn->nId; j++){
-        if( pColumn->a[j].idx==i ) break;
+    /* Push onto the stack, data for all columns of the new entry, beginning
+    ** with the first column.
+    */
+    for(i=0; i<pTab->nCol; i++){
+      if( i==pTab->iPKey ){
+        /* The value of the INTEGER PRIMARY KEY column is always a NULL.
+        ** Whenever this column is read, the record number will be substituted
+        ** in its place.  So will fill this column with a NULL to avoid
+        ** taking up data space with information that will never be used. */
+        sqliteVdbeAddOp(v, OP_String, 0, 0);
+        continue;
+      }
+      if( pColumn==0 ){
+        j = i;
+      }else{
+        for(j=0; j<pColumn->nId; j++){
+          if( pColumn->a[j].idx==i ) break;
+        }
+      }
+      if( pColumn && j>=pColumn->nId ){
+        sqliteVdbeAddOp(v, OP_String, 0, 0);
+        sqliteVdbeChangeP3(v, -1, pTab->aCol[i].zDflt, P3_STATIC);
+      }else if( srcTab>=0 ){
+        sqliteVdbeAddOp(v, OP_Column, srcTab, j); 
+      }else{
+        sqliteExprCode(pParse, pList->a[j].pExpr);
       }
     }
-    if( pColumn && j>=pColumn->nId ){
-      sqliteVdbeAddOp(v, OP_String, 0, 0);
-      sqliteVdbeChangeP3(v, -1, pTab->aCol[i].zDflt, P3_STATIC);
-    }else if( srcTab>=0 ){
-      sqliteVdbeAddOp(v, OP_Column, srcTab, j); 
-    }else{
-      sqliteExprCode(pParse, pList->a[j].pExpr);
+
+    /* Generate code to check constraints and generate index keys and
+    ** do the insertion.
+    */
+    sqliteGenerateConstraintChecks(pParse, pTab, base, 0,0,0,onError,endOfLoop);
+    sqliteCompleteInsertion(pParse, pTab, base, 0,0,0);
+
+    /* Update the count of rows that are inserted
+    */
+    if( (db->flags & SQLITE_CountRows)!=0 && !pParse->trigStack){
+      sqliteVdbeAddOp(v, OP_AddImm, 1, 0);
     }
   }
 
-  /* Generate code to check constraints and generate index keys and
-  ** do the insertion.
-  */
-  endOfLoop = sqliteVdbeMakeLabel(v);
-  sqliteGenerateConstraintChecks(pParse, pTab, base, 0,0,0, onError, endOfLoop);
-  sqliteCompleteInsertion(pParse, pTab, base, 0,0,0);
+  if( row_triggers_exist ){
+    /* Close all tables opened */
+    if( !pTab->pSelect ){
+      sqliteVdbeAddOp(v, OP_Close, base, 0);
+      for(idx=1, pIdx=pTab->pIndex; pIdx; pIdx=pIdx->pNext, idx++){
+        sqliteVdbeAddOp(v, OP_Close, idx+base, 0);
+      }
+    }
 
-  /* If inserting from a SELECT, keep a count of the number of
-  ** rows inserted.
-  */
-  if( srcTab>=0 && (db->flags & SQLITE_CountRows)!=0 ){
-    sqliteVdbeAddOp(v, OP_AddImm, 1, 0);
+    /* Code AFTER triggers */
+    if( sqliteCodeRowTrigger(pParse, TK_INSERT, 0, TK_AFTER, pTab, newIdx, -1, 
+          onError, endOfLoop) ){
+      goto insert_cleanup;
+    }
   }
 
   /* The bottom of the loop, if the data source is a SELECT statement
@@ -273,28 +371,31 @@ void sqliteInsert(
     sqliteVdbeResolveLabel(v, iBreak);
     sqliteVdbeAddOp(v, OP_Close, srcTab, 0);
   }
-  sqliteVdbeAddOp(v, OP_Close, base, 0);
-  for(idx=1, pIdx=pTab->pIndex; pIdx; pIdx=pIdx->pNext, idx++){
-    sqliteVdbeAddOp(v, OP_Close, idx+base, 0);
+
+  if( !row_triggers_exist ){
+    /* Close all tables opened */
+    sqliteVdbeAddOp(v, OP_Close, base, 0);
+    for(idx=1, pIdx=pTab->pIndex; pIdx; pIdx=pIdx->pNext, idx++){
+      sqliteVdbeAddOp(v, OP_Close, idx+base, 0);
+    }
   }
+
   sqliteEndWriteOperation(pParse);
 
   /*
   ** Return the number of rows inserted.
   */
-  if( db->flags & SQLITE_CountRows ){
+  if( db->flags & SQLITE_CountRows && !pParse->trigStack ){
     sqliteVdbeAddOp(v, OP_ColumnCount, 1, 0);
     sqliteVdbeAddOp(v, OP_ColumnName, 0, 0);
     sqliteVdbeChangeP3(v, -1, "rows inserted", P3_STATIC);
-    if( srcTab<0 ){
-      sqliteVdbeAddOp(v, OP_Integer, 1, 0);
-    }
     sqliteVdbeAddOp(v, OP_Callback, 1, 0);
   }
 
 insert_cleanup:
   if( pList ) sqliteExprListDelete(pList);
   if( pSelect ) sqliteSelectDelete(pSelect);
+  if ( zTab ) sqliteFree(zTab);
   sqliteIdListDelete(pColumn);
 }
 
@@ -396,7 +497,7 @@ void sqliteGenerateConstraintChecks(
   int iCur;
   Index *pIdx;
   int seenReplace = 0;
-  int jumpInst;
+  int jumpInst1, jumpInst2;
   int contAddr;
   int hasTwoRecnos = (isUpdate && recnoChng);
 
@@ -424,7 +525,7 @@ void sqliteGenerateConstraintChecks(
       onError = OE_Abort;
     }
     sqliteVdbeAddOp(v, OP_Dup, nCol-1-i, 1);
-    addr = sqliteVdbeAddOp(v, OP_NotNull, 0, 0);
+    addr = sqliteVdbeAddOp(v, OP_NotNull, 1, 0);
     switch( onError ){
       case OE_Rollback:
       case OE_Abort:
@@ -450,8 +551,13 @@ void sqliteGenerateConstraintChecks(
 
   /* Test all CHECK constraints
   */
+  /**** TBD ****/
 
-  /* Test all UNIQUE constraints.  Add index records as we go.
+  /* If we have an INTEGER PRIMARY KEY, make sure the primary key
+  ** of the new record does not previously exist.  Except, if this
+  ** is an UPDATE and the primary key is not changing, that is OK.
+  ** Also, if the conflict resolution policy is REPLACE, then we
+  ** can skip this test.
   */
   if( (recnoChng || !isUpdate) && pTab->iPKey>=0 ){
     onError = pTab->keyConf;
@@ -462,8 +568,13 @@ void sqliteGenerateConstraintChecks(
       if( onError==OE_Default ) onError = OE_Abort;
     }
     if( onError!=OE_Replace ){
+      if( isUpdate ){
+        sqliteVdbeAddOp(v, OP_Dup, nCol+1, 1);
+        sqliteVdbeAddOp(v, OP_Dup, nCol+1, 1);
+        jumpInst1 = sqliteVdbeAddOp(v, OP_Eq, 0, 0);
+      }
       sqliteVdbeAddOp(v, OP_Dup, nCol, 1);
-      jumpInst = sqliteVdbeAddOp(v, OP_NotExists, base, 0);
+      jumpInst2 = sqliteVdbeAddOp(v, OP_NotExists, base, 0);
       switch( onError ){
         case OE_Rollback:
         case OE_Abort:
@@ -479,13 +590,19 @@ void sqliteGenerateConstraintChecks(
         default: assert(0);
       }
       contAddr = sqliteVdbeCurrentAddr(v);
-      sqliteVdbeChangeP2(v, jumpInst, contAddr);
+      sqliteVdbeChangeP2(v, jumpInst2, contAddr);
       if( isUpdate ){
+        sqliteVdbeChangeP2(v, jumpInst1, contAddr);
         sqliteVdbeAddOp(v, OP_Dup, nCol+1, 1);
         sqliteVdbeAddOp(v, OP_MoveTo, base, 0);
       }
     }
   }
+
+  /* Test all UNIQUE constraints by creating entries for each UNIQUE
+  ** index and making sure that duplicate entries do not already exist.
+  ** Add the new records to the indices as we go.
+  */
   extra = 0;
   for(extra=(-1), iCur=0, pIdx=pTab->pIndex; pIdx; pIdx=pIdx->pNext, iCur++){
     if( aIdxUsed && aIdxUsed[iCur]==0 ) continue;
@@ -499,7 +616,7 @@ void sqliteGenerateConstraintChecks(
         sqliteVdbeAddOp(v, OP_Dup, i+extra+nCol-idx, 1);
       }
     }
-    sqliteVdbeAddOp(v, OP_MakeIdxKey, pIdx->nColumn, 0);
+    jumpInst1 = sqliteVdbeAddOp(v, OP_MakeIdxKey, pIdx->nColumn, 0);
     onError = pIdx->onError;
     if( onError==OE_None ) continue;
     if( overrideError!=OE_Default ){
@@ -509,7 +626,7 @@ void sqliteGenerateConstraintChecks(
       if( onError==OE_Default ) onError = OE_Abort;
     }
     sqliteVdbeAddOp(v, OP_Dup, extra+nCol+1+hasTwoRecnos, 1);
-    jumpInst = sqliteVdbeAddOp(v, OP_IsUnique, base+iCur+1, 0);
+    jumpInst2 = sqliteVdbeAddOp(v, OP_IsUnique, base+iCur+1, 0);
     switch( onError ){
       case OE_Rollback:
       case OE_Abort:
@@ -519,12 +636,12 @@ void sqliteGenerateConstraintChecks(
       }
       case OE_Ignore: {
         assert( seenReplace==0 );
-        sqliteVdbeAddOp(v, OP_Pop, nCol+extra+2+hasTwoRecnos, 0);
+        sqliteVdbeAddOp(v, OP_Pop, nCol+extra+3+hasTwoRecnos, 0);
         sqliteVdbeAddOp(v, OP_Goto, 0, ignoreDest);
         break;
       }
       case OE_Replace: {
-        sqliteGenerateRowDelete(v, pTab, base);
+        sqliteGenerateRowDelete(v, pTab, base, 0);
         if( isUpdate ){
           sqliteVdbeAddOp(v, OP_Dup, nCol+extra+1+hasTwoRecnos, 1);
           sqliteVdbeAddOp(v, OP_MoveTo, base, 0);
@@ -535,7 +652,10 @@ void sqliteGenerateConstraintChecks(
       default: assert(0);
     }
     contAddr = sqliteVdbeCurrentAddr(v);
-    sqliteVdbeChangeP2(v, jumpInst, contAddr);
+#if NULL_DISTINCT_FOR_UNIQUE
+    sqliteVdbeChangeP2(v, jumpInst1, contAddr);
+#endif
+    sqliteVdbeChangeP2(v, jumpInst2, contAddr);
   }
 }
 
@@ -571,7 +691,7 @@ void sqliteCompleteInsertion(
     sqliteVdbeAddOp(v, OP_IdxPut, base+i+1, 0);
   }
   sqliteVdbeAddOp(v, OP_MakeRecord, pTab->nCol, 0);
-  sqliteVdbeAddOp(v, OP_PutIntKey, base, 0);
+  sqliteVdbeAddOp(v, OP_PutIntKey, base, pParse->trigStack?0:1);
   if( isUpdate && recnoChng ){
     sqliteVdbeAddOp(v, OP_Pop, 1, 0);
   }
