@@ -17,22 +17,6 @@
 #include "sqliteInt.h"
 #include "os.h"
 
-#ifndef OS_UNIX
-# ifndef OS_WIN
-#  if defined(_WIN32) || defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__) || defined(__BORLANDC__)
-#    define OS_WIN 1
-#    define OS_UNIX 0
-#  else
-#    define OS_WIN 0
-#    define OS_UNIX 1
-#  endif
-# else
-#  define OS_UNIX 0
-# endif
-#endif
-#ifndef OS_WIN
-# define OS_WIN 0
-#endif
 #if OS_UNIX
 # include <unistd.h>
 # include <fcntl.h>
@@ -451,8 +435,10 @@ int sqliteOsTempFileName(char *zBuf){
   int i, j;
   char zTempPath[SQLITE_TEMPNAME_SIZE];
   GetTempPath(SQLITE_TEMPNAME_SIZE-30, zTempPath);
+  for(i=strlen(zTempPath); i>0 && zTempPath[i-1]=='\\'; i--){}
+  zTempPath[i] = 0;
   for(;;){
-    sprintf(zBuf, "%s/sqlite_", zTempPath);
+    sprintf(zBuf, "%s\\sqlite_", zTempPath);
     j = strlen(zBuf);
     for(i=0; i<15; i++){
       int n = sqliteRandomByte() % sizeof(zChars);
